<?php

include_once __DIR__.'/constants.php';
include_once __DIR__.'/transactionHelper.php';


/**
 * formats the given string to be only 1 line
 * @param $output string the output
 * @return mixed|string
 */
function formatOutput($output) {

    global $responseNewLineString;

    $formattedOutput = $output;
    # backend requires output to be a string... so convert it if needed
    if (count($formattedOutput) === 0) {
        $formattedOutput = '';
    } else if (is_array($output)) {
        $formattedOutput = join($responseNewLineString, $formattedOutput);
        debug('output: ' . $formattedOutput);
    } else {
        //should be just 1 line string
//        $formattedOutput = str_replace("\n", $responseNewLineString, $formattedOutput);

        //with runner version 2.4 the runner uses \r\n already
        $formattedOutput = $output;
        debug('output: ' . $formattedOutput);
    }

    return $formattedOutput;
}

/**
 * creates (if not exists) the dirs used for the test
 * @param string $currentRootDirName the current root dir for this iteration
 * @param $tempUserDirName string the dir name to use/create
 * @return array|bool
 */
function ensureDirs($currentRootDirName, $tempUserDirName)
{

    global $s_config_workingDirFullPath;
    global $config;
    global $s_tempDirName;
    global $s_fullWorkingDirPath;
    global $timeZone;
    global $status_code_FileSystemIssue;
    global $isDebug;


    # this is the root path for all tests but every iteration has its own root folder in that absolute root folder
    $config_workingDirFullPath = $config[$s_config_workingDirFullPath];

    #make the path end with dir separator
    if (!endsWith($config_workingDirFullPath, DIRECTORY_SEPARATOR)) {
        $config_workingDirFullPath = $config_workingDirFullPath . DIRECTORY_SEPARATOR;
    }

    if (!endsWith($currentRootDirName, DIRECTORY_SEPARATOR)) {
        $currentRootDirName = $currentRootDirName . DIRECTORY_SEPARATOR;
    }


    #date_default_timezone_set($timeZone);
    #$workingDirTempName = date_timestamp_get(date_create());
    # timestamp is too bad... we use the db transaction id/key becasue we user microtime and a request distinction number
    # which should be unique for EVERY request
    $workingDirTempName = $tempUserDirName;


    $fullWorkingDirPath = $config_workingDirFullPath . $currentRootDirName . $workingDirTempName;

    $returnArray = array(
        $s_tempDirName => $workingDirTempName, //the name of the temp directory
        $s_fullWorkingDirPath => $fullWorkingDirPath, //the dir where the working files will be places
    );


    if (file_exists($fullWorkingDirPath) && is_dir($fullWorkingDirPath)) {
        //working dir already exists... ok
        return $returnArray;
    }

    # see https://stackoverflow.com/questions/3997641/why-cant-php-create-a-directory-with-777-permissions
    # why 0777 is not working...
    # not needed anymore
    # $old = umask(0);
    # create the temporarily dir for the files
    # TOOD maybe somehow make this better than 777 (because of firejail)
    # use 777 so that every user can access e.g. if we use firejail with starts as another user...
    if (mkdir($fullWorkingDirPath, 0755, TRUE) === FALSE) {

        # we output this in the calling method
        if ($isDebug) {
            # os this will throw in the frontend but this is ok for debug...
            output($status_code_FileSystemIssue, 'could not create directory: ' . $fullWorkingDirPath);
        }
        #else {
        #    output($status_code_FileSystemIssue, 'could not create directory');
        #}

        #umask($old);
        return FALSE;
    }

    #umask($old);
    return $returnArray;
}

/**
 * creates all given files inside a temp dir
 * @param array $allFiles all files to create
 * Format: array[int] => {fileName: string, fileContent: string}
 * @global string $s_config_workingDirFullPath the config working dir path
 * @param string $fullWorkingDirPath the path (dir) where to put the files
 * @global string $config the config
 * @return bool FALSE: could not create files
 */
function createFiles($allFiles, $fullWorkingDirPath)
{

    global $s_fileName;
    global $s_fileContent;
    global $status_code_FileSystemIssue;
    global $isDebug;

    if ($isDebug) {
        debug("using working dir: " . $fullWorkingDirPath);
    }

    # now create all files for the test
    $count = 0;
    foreach ($allFiles as $file) {
        if (file_put_contents($fullWorkingDirPath . DIRECTORY_SEPARATOR . $file[$s_fileName], $file[$s_fileContent]) === FALSE) {
            output($status_code_FileSystemIssue, "could not create file nr. " . $count);
            return FALSE;
        }
        $count++;
    }

    return TRUE;
}

/**
 * @param array $allAssets all test assets
 * Format: array[int] => {fileName: string, fileContent: byte[]}
 * @param string $fullWorkingDirPath the path (dir) where to put the files
 * @return bool FALSE: could not create files
 */
function createTestAssets($allAssets, $fullWorkingDirPath)
{

    # do not output something here because we use this in the test case loop
    global $s_fileName;
    global $s_fileContent;
    global $status_code_FileSystemIssue;

    # now create all files for the test
    $count = 0;
    foreach ($allAssets as $file) {

        $content = base64_decode($file[$s_fileContent]);

        if ($content === FALSE) {
            //could not decode byte array...
            #output($status_code_FileSystemIssue, "could not decode create file nr. " . $count);
            return FALSE;
        }

        if (file_put_contents($fullWorkingDirPath . DIRECTORY_SEPARATOR . $file[$s_fileName], $content) === FALSE) {
            #output($status_code_FileSystemIssue, "could not create file nr. " . $count);
            return FALSE;
        }

        $count++;
    }
    return TRUE;
}

/**
 * removes the given dir with all content
 * taken from http://stackoverflow.com/questions/3349753/delete-directory-with-files-in-it
 * @param string $dirPath
 */
function deleteDir($dirPath)
{
    global $status_code_FileSystemIssue;

    if (!is_dir($dirPath)) {
        //this can happen if two different php processes try to delete the same dir??
        //this should not fail the test
        //output($status_code_FileSystemIssue, "could not delete temp dir");
        warn("could not delete temp dir (!is_dir(dir)): '" . $dirPath . "'");
        //don't exit() here because the transaction is not closed yet
        //return;
    }
    if (substr($dirPath, strlen($dirPath) - 1, 1) != '/') {
        $dirPath .= '/';
    }
    $files = glob($dirPath . '*', GLOB_MARK);
    foreach ($files as $file) {
        if (is_dir($file)) {
            deleteDir($file);
        } else {
            unlink($file);
        }
    }
    rmdir($dirPath);
}

/**
 * add the message to the timing debug log
 * @param $message String to add
 */
function addToDebugTimings($message)
{
    global $debugTimings;
    #see https://stackoverflow.com/questions/17909871/getting-date-format-m-d-y-his-u-from-milliseconds
    $t = microtime(true);
    $micro = sprintf("%06d",($t - floor($t)) * 1000000);
    $d = new DateTime( date('Y-m-d H:i:s.'.$micro, $t) );
    array_push($debugTimings, "[" . $d->format("Y-m-d H:i:s.u") . "] " . $message);
}